@@ -3,11 +3,12 @@ use crate::batch::Batch;
 use crate::frame::response::result::Row;
 use crate::frame::value::ValueList;
 use crate::query::Query;
-use crate::routing::hash3_x64_128;
+use crate::routing::Token;
 use crate::statement::Consistency;
 use crate::tracing::TracingInfo;
 use crate::transport::connection::BatchResult;
 use crate::transport::errors::{BadKeyspaceName, BadQuery, DbError, QueryError};
+use crate::transport::partitioner::{Murmur3Partitioner, Partitioner, PartitionerName};
 use crate::transport::topology::Strategy::SimpleStrategy;
 use crate::transport::topology::{CollectionType, ColumnKind, CqlType, NativeType};
 use crate::QueryResult;
@@ -200,16 +201,18 @@ async fn test_prepared_statement() {
             .unwrap()
             .rows
             .unwrap();
-        let token: i64 = rs.first().unwrap().columns[0]
-            .as_ref()
-            .unwrap()
-            .as_bigint()
-            .unwrap();
-        let expected_token = hash3_x64_128(
-            &prepared_statement
+        let token = Token {
+            value: rs.first().unwrap().columns[0]
+                .as_ref()
+                .unwrap()
+                .as_bigint()
+                .unwrap(),
+        };
+        let expected_token = Murmur3Partitioner::hash(
+            prepared_statement
                 .compute_partition_key(&serialized_values)
                 .unwrap(),
-        ) as i64;
+        );
 
         assert_eq!(token, expected_token)
     }
@@ -220,16 +223,18 @@ async fn test_prepared_statement() {
             .unwrap()
             .rows
             .unwrap();
-        let token: i64 = rs.first().unwrap().columns[0]
-            .as_ref()
-            .unwrap()
-            .as_bigint()
-            .unwrap();
-        let expected_token = hash3_x64_128(
-            &prepared_complex_pk_statement
+        let token = Token {
+            value: rs.first().unwrap().columns[0]
+                .as_ref()
+                .unwrap()
+                .as_bigint()
+                .unwrap(),
+        };
+        let expected_token = Murmur3Partitioner::hash(
+            prepared_complex_pk_statement
                 .compute_partition_key(&serialized_values)
                 .unwrap(),
-        ) as i64;
+        );
 
         assert_eq!(token, expected_token)
     }
@@ -473,16 +478,18 @@ async fn test_token_calculation() {
             .unwrap()
             .rows
             .unwrap();
-        let token: i64 = rs.first().unwrap().columns[0]
-            .as_ref()
-            .unwrap()
-            .as_bigint()
-            .unwrap();
-        let expected_token = hash3_x64_128(
-            &prepared_statement
+        let token = Token {
+            value: rs.first().unwrap().columns[0]
+                .as_ref()
+                .unwrap()
+                .as_bigint()
+                .unwrap(),
+        };
+        let expected_token = Murmur3Partitioner::hash(
+            prepared_statement
                 .compute_partition_key(&serialized_values)
                 .unwrap(),
-        ) as i64;
+        );
         assert_eq!(token, expected_token)
     }
 }
@@ -1694,3 +1701,56 @@ async fn test_named_bind_markers() {
         assert!(session.execute(&prepared, &wrongmap).await.is_err());
     }
 }
+
+#[tokio::test]
+async fn test_prepared_partitioner() {
+    let uri = std::env::var("SCYLLA_URI").unwrap_or_else(|_| "127.0.0.1:9042".to_string());
+    let session = SessionBuilder::new().known_node(uri).build().await.unwrap();
+    let ks = unique_name();
+
+    session.query(format!("CREATE KEYSPACE IF NOT EXISTS {} WITH REPLICATION = {{'class' : 'SimpleStrategy', 'replication_factor' : 1}}", ks), &[]).await.unwrap();
+    session.use_keyspace(ks, false).await.unwrap();
+
+    session
+        .query("CREATE TABLE IF NOT EXISTS t1 (a int primary key)", &[])
+        .await
+        .unwrap();
+
+    session.await_schema_agreement().await.unwrap();
+    session.refresh_metadata().await.unwrap();
+
+    let prepared_statement_for_main_table = session
+        .prepare("INSERT INTO t1 (a) VALUES (?)")
+        .await
+        .unwrap();
+
+    assert_eq!(
+        prepared_statement_for_main_table.get_partitioner_name(),
+        &PartitionerName::Murmur3
+    );
+
+    if option_env!("CDC") == Some("disabled") {
+        return;
+    }
+
+    session
+        .query(
+            "CREATE TABLE IF NOT EXISTS t2 (a int primary key) WITH cdc = {'enabled':true}",
+            &[],
+        )
+        .await
+        .unwrap();
+
+    session.await_schema_agreement().await.unwrap();
+    session.refresh_metadata().await.unwrap();
+
+    let prepared_statement_for_cdc_log = session
+        .prepare("SELECT a FROM t2_scylla_cdc_log WHERE \"cdc$stream_id\" = ?")
+        .await
+        .unwrap();
+
+    assert_eq!(
+        prepared_statement_for_cdc_log.get_partitioner_name(),
+        &PartitionerName::CDC
+    );
+}
